import os
import shutil
import tempfile
import unittest
from zope.component import queryUtility
from zope.interface.verify import verifyObject, verifyClass
from waeup.sirp.interfaces import ICSVExporter
from waeup.sirp.testing import SIRPUnitTestLayer
from waeup.sirp.university import (
    FacultiesContainer, Faculty, Department, Course,
    )
from waeup.sirp.university.export import (
    FacultyExporter, DepartmentExporter, CourseExporter,
    )

class FacultyExporterTest(unittest.TestCase):

    layer = SIRPUnitTestLayer

    def setUp(self):
        self.workdir = tempfile.mkdtemp()
        self.outfile = os.path.join(self.workdir, 'myoutput.csv')
        return

    def tearDown(self):
        shutil.rmtree(self.workdir)
        return

    def test_ifaces(self):
        # make sure we fullfill interface contracts
        obj = FacultyExporter()
        verifyObject(ICSVExporter, obj)
        verifyClass(ICSVExporter, FacultyExporter)
        return

    def test_get_as_utility(self):
        # we can get a faculty exporter as utility
        result = queryUtility(ICSVExporter, name="faculties")
        self.assertTrue(result is not None)
        return

    def test_export(self):
        # we can export a set of faculties
        fac = Faculty('Faculty of Cheese', 'faculty', 'F1')
        exporter = FacultyExporter()
        exporter.export([fac], self.outfile)
        result = open(self.outfile, 'rb').read()
        self.assertEqual(
            result,
            'code,title,title_prefix\r\n'
            'F1,Faculty of Cheese,faculty\r\n'
            )
        return

    def test_export_to_string(self):
        # we can export a set of faculties to a string.
        fac = Faculty('Faculty of Cheese', 'faculty', 'F1')
        exporter = FacultyExporter()
        result = exporter.export([fac], filepath=None)
        self.assertEqual(
            result,
            'code,title,title_prefix\r\n'
            'F1,Faculty of Cheese,faculty\r\n'
            )
        return

    def test_export_all(self):
        # we can export all faculties in a site
        container = FacultiesContainer()
        site = {'faculties':container}
        fac1 = Faculty('Faculty of Cheese', 'faculty', 'F1')
        fac2 = Faculty('Centre of Onion', 'centre', 'F2')
        container.addFaculty(fac1)
        container.addFaculty(fac2)
        exporter = FacultyExporter()
        exporter.export_all(site, self.outfile)
        result = open(self.outfile, 'rb').read()
        self.assertEqual(
            result,
            'code,title,title_prefix\r\n'
            'F1,Faculty of Cheese,faculty\r\n'
            'F2,Centre of Onion,centre\r\n'
            )
        return

    def test_export_all_to_string(self):
        # we can export all faculties in a site to a string
        container = FacultiesContainer()
        site = {'faculties':container}
        fac1 = Faculty('Faculty of Cheese', 'faculty', 'F1')
        fac2 = Faculty('Centre of Onion', 'centre', 'F2')
        container.addFaculty(fac1)
        container.addFaculty(fac2)
        exporter = FacultyExporter()
        result = exporter.export_all(site, filepath=None)
        self.assertEqual(
            result,
            'code,title,title_prefix\r\n'
            'F1,Faculty of Cheese,faculty\r\n'
            'F2,Centre of Onion,centre\r\n'
            )
        return

class DepartmentExporterTest(unittest.TestCase):
    # Tests for DepartmentExporter

    layer = SIRPUnitTestLayer

    def setUp(self):
        self.workdir = tempfile.mkdtemp()
        self.outfile = os.path.join(self.workdir, 'myoutput.csv')
        # create some departments in a fake site
        container = FacultiesContainer()
        self.site = {'faculties':container}
        self.fac1 = Faculty('Faculty of Cheese', 'faculty', 'F1')
        self.fac2 = Faculty('Centre of Onion', 'centre', 'F2')
        container.addFaculty(self.fac1)
        container.addFaculty(self.fac2)
        self.dept1 = Department('Department of Cheddar', 'department', 'D1')
        self.dept2 = Department('Institue of Gouda', 'institute', 'D2')
        self.dept3 = Department('Department of Rings', 'department', 'D3')
        self.fac1.addDepartment(self.dept1)
        self.fac1.addDepartment(self.dept2)
        self.fac2.addDepartment(self.dept3)
        return

    def tearDown(self):
        shutil.rmtree(self.workdir)
        return

    def test_ifaces(self):
        # make sure we fullfill interface contracts
        obj = DepartmentExporter()
        verifyObject(ICSVExporter, obj)
        verifyClass(ICSVExporter, DepartmentExporter)
        return

    def test_get_as_utility(self):
        # we can get a department exporter as utility
        result = queryUtility(ICSVExporter, name="departments")
        self.assertTrue(result is not None)
        return

    def test_export(self):
        # we can export an iterable of departments
        exporter = DepartmentExporter()
        exporter.export([self.dept1], self.outfile)
        result = open(self.outfile, 'rb').read()
        self.assertEqual(
            result,
            'code,faculty_code,title,title_prefix\r\n'
            'D1,F1,Department of Cheddar,department\r\n'
            )
        return

    def test_export_to_string(self):
        # we can export an iterable of departments to a string.
        exporter = DepartmentExporter()
        result = exporter.export([self.dept1, self.dept2], filepath=None)
        self.assertEqual(
            result,
            'code,faculty_code,title,title_prefix\r\n'
            'D1,F1,Department of Cheddar,department\r\n'
            'D2,F1,Institue of Gouda,institute\r\n'
            )
        return

    def test_export_all(self):
        # we can export all depts in a site
        exporter = DepartmentExporter()
        exporter.export_all(self.site, self.outfile)
        result = open(self.outfile, 'rb').read()
        self.assertEqual(
            result,
            'code,faculty_code,title,title_prefix\r\n'
            'D1,F1,Department of Cheddar,department\r\n'
            'D2,F1,Institue of Gouda,institute\r\n'
            'D3,F2,Department of Rings,department\r\n'
            )
        return

    def test_export_all_to_string(self):
        # we can export all depts in a site to a string
        exporter = DepartmentExporter()
        result = exporter.export_all(self.site, filepath=None)
        self.assertEqual(
            result,
            'code,faculty_code,title,title_prefix\r\n'
            'D1,F1,Department of Cheddar,department\r\n'
            'D2,F1,Institue of Gouda,institute\r\n'
            'D3,F2,Department of Rings,department\r\n'
            )
        return

class CourseExporterTest(unittest.TestCase):
    # Tests for CourseExporter

    layer = SIRPUnitTestLayer

    def setUp(self):
        self.workdir = tempfile.mkdtemp()
        self.outfile = os.path.join(self.workdir, 'myoutput.csv')
        # create some departments and courses in a fake site
        container = FacultiesContainer()
        self.site = {'faculties':container}
        self.fac = Faculty('Faculty of Cheese', 'faculty', 'F1')
        container.addFaculty(self.fac)
        self.dept1 = Department('Department of Cheddar', 'department', 'D1')
        self.dept2 = Department('Institue of Gouda', 'institute', 'D2')
        self.fac.addDepartment(self.dept1)
        self.fac.addDepartment(self.dept2)
        self.course1 = Course('Cheese Basics', 'C1')
        self.course2 = Course('Advanced Cheese Making', 'C2')
        self.course3 = Course('Selling Cheese', 'C3')
        self.dept1.courses[u'C1'] = self.course1
        self.dept1.courses['C2'] = self.course2
        self.dept2.courses[u'C3'] = self.course3
        return

    def tearDown(self):
        shutil.rmtree(self.workdir)
        return

    def test_ifaces(self):
        # make sure we fullfill interface contracts
        obj = CourseExporter()
        verifyObject(ICSVExporter, obj)
        verifyClass(ICSVExporter, CourseExporter)
        return

    def test_get_as_utility(self):
        # we can get a course exporter as utility
        result = queryUtility(ICSVExporter, name="courses")
        self.assertTrue(result is not None)
        return

    def test_export(self):
        # we can export an iterable of courses
        exporter = CourseExporter()
        exporter.export([self.course1], self.outfile)
        result = open(self.outfile, 'rb').read()
        self.assertEqual(
            result,
            'code,faculty_code,department_code,title,credits,passmark,semester\r\n'
            'C1,F1,,Cheese Basics,0,40,1\r\n'
            )
        return

    def test_export_to_string(self):
        # we can export an iterable of courses to a string.
        exporter = CourseExporter()
        result = exporter.export([self.course1, self.course2], filepath=None)
        self.assertEqual(
            result,
            'code,faculty_code,department_code,title,credits,passmark,semester\r\n'
            'C1,F1,,Cheese Basics,0,40,1\r\n'
            'C2,F1,,Advanced Cheese Making,0,40,1\r\n'
            )
        return

    def test_export_all(self):
        # we can export all courses in a site
        exporter = CourseExporter()
        exporter.export_all(self.site, self.outfile)
        result = open(self.outfile, 'rb').read()
        self.assertEqual(
            result,
            'code,faculty_code,department_code,title,credits,passmark,semester\r\n'
            'C1,F1,,Cheese Basics,0,40,1\r\n'
            'C2,F1,,Advanced Cheese Making,0,40,1\r\n'
            'C3,F1,,Selling Cheese,0,40,1\r\n'
            )
        return

    def test_export_all_to_string(self):
        # we can export all courses in a site to a string
        exporter = CourseExporter()
        result = exporter.export_all(self.site, filepath=None)
        self.assertEqual(
            result,
            'code,faculty_code,department_code,title,credits,passmark,semester\r\n'
            'C1,F1,,Cheese Basics,0,40,1\r\n'
            'C2,F1,,Advanced Cheese Making,0,40,1\r\n'
            'C3,F1,,Selling Cheese,0,40,1\r\n'
            )
        return
